@@ -138,10 +138,7 @@ class BaseList(list):
         return super(BaseList, self).__setitem__(key, value)
 
     def __delitem__(self, key, *args, **kwargs):
-        if isinstance(key, slice):
-            self._mark_as_changed()
-        else:
-            self._mark_as_changed(key)
+        self._mark_as_changed()
         return super(BaseList, self).__delitem__(key)
 
     def __setslice__(self, *args, **kwargs):
@@ -432,7 +429,7 @@ class StrictDict(object):
     def __eq__(self, other):
         return list(self.items()) == list(other.items())
 
-    def __neq__(self, other):
+    def __ne__(self, other):
         return list(self.items()) != list(other.items())
 
     @classmethod
